@@ -1,13 +1,13 @@
-#[cfg(any(feature = "doc", feature = "use_gpu"))]
+#[cfg(feature = "use_gpu")]
 mod ocl;
 
-#[cfg(any(feature = "doc", feature = "use_gpu"))]
+#[cfg(feature = "use_gpu")]
 pub use self::ocl::*;
 
-#[cfg(any(feature = "doc", not(feature = "use_gpu")))]
+#[cfg(not(eature = "use_gpu"))]
 mod fallback;
 
-#[cfg(any(feature = "doc", not(feature = "use_gpu")))]
+#[cfg(not(eature = "use_gpu"))]
 pub use self::fallback::*;
 
 use numbers::*;
