@@ -14,52 +14,6 @@ use std::slice::{Chunks, ChunksMut};
 use std::sync::{Arc, Mutex};
 use time;
 
-/// Holds parameters which specify how multiple cores are used
-/// to execute an operation.
-#[derive(Debug, Copy, Clone)]
-#[repr(C)]
-pub struct MultiCoreSettings {
-    /// All operations will be limited to not create more threads than specified here
-    pub core_limit: usize,
-}
-
-lazy_static! {
-    static ref NUMBER_OF_CORES: usize = num_cpus::get();
-}
-
-impl MultiCoreSettings {
-    /// Creates multi core settings with default values
-    pub fn default() -> MultiCoreSettings {
-        Self::single_threaded()
-    }
-
-    /// Creates multi core settings so that no thread will be spawned.
-    pub fn single_threaded() -> MultiCoreSettings {
-        Self::new(1)
-    }
-
-    /// Creates multi core so that threads will be spawned if this appears to be beneficial.
-    pub fn parallel() -> MultiCoreSettings {
-        if *NUMBER_OF_CORES == 1 {
-            Self::new(1)
-        } else if *NUMBER_OF_CORES == 2 {
-            // If two cores are available then use both of them if multi-threading is requested
-            Self::new(2)
-        } else {
-            // Half because we assume hyper threading and that we will keep a core so busy
-            // that hyper threading isn't of any use
-            Self::new(*NUMBER_OF_CORES / 2)
-        }
-    }
-
-    /// Creates multi core settings with the given values.
-    pub fn new(core_limit: usize) -> MultiCoreSettings {
-        MultiCoreSettings {
-            core_limit: if core_limit >= 1 { core_limit } else { 1 },
-        }
-    }
-}
-
 /// Calibration information which determines when multi threaded code will be used.
 #[derive(Debug, PartialEq)]
 struct Calibration {
@@ -210,6 +164,11 @@ fn attempt_calibrate(number_of_cores: usize) -> Result<Calibration, u32> {
     })
 }
 
+static MED_DUAL_CORE_DEFAULT: usize = 50_000;
+static MED_MULTI_CORE_DEFAULT: usize = 100_000;
+static LARGE_DUAL_CORE_DEFAULT: usize = 20_000;
+static LARGE_MULTI_CORE_DEFAULT: usize = 30_000;
+
 fn calibrate(number_of_cores: usize) -> Calibration {
     let start = time::precise_time_s();
     match attempt_calibrate(number_of_cores) {
@@ -218,10 +177,10 @@ fn calibrate(number_of_cores: usize) -> Calibration {
             calibration
         }
         Err(err_code) => Calibration {
-            med_dual_core_threshold: 50_000,
-            med_multi_core_threshold: 100_000,
-            large_dual_core_threshold: 20_000,
-            large_multi_core_threshold: 30_000,
+            med_dual_core_threshold: MED_DUAL_CORE_DEFAULT,
+            med_multi_core_threshold: MED_MULTI_CORE_DEFAULT,
+            large_dual_core_threshold: LARGE_DUAL_CORE_DEFAULT,
+            large_multi_core_threshold: LARGE_MULTI_CORE_DEFAULT,
             duration_cal_routine: time::precise_time_s() - start,
             cal_routine_result_code: err_code,
         },
@@ -230,6 +189,94 @@ fn calibrate(number_of_cores: usize) -> Calibration {
 
 lazy_static! {
     static ref CALIBRATION: Calibration = calibrate(*NUMBER_OF_CORES);
+    static ref NUMBER_OF_CORES: usize = num_cpus::get();
+}
+
+/// Holds parameters which specify how multiple cores are used
+/// to execute an operation.
+#[derive(Debug, Copy, Clone)]
+#[repr(C)]
+pub struct MultiCoreSettings {
+    /// All operations will be limited to not create more threads than specified here
+    pub core_limit: usize,
+    pub med_dual_core_threshold: usize,
+    pub med_multi_core_threshold: usize,
+    pub large_dual_core_threshold: usize,
+    pub large_multi_core_threshold: usize,
+}
+
+impl MultiCoreSettings {
+    /// Creates multi core settings with default values
+    pub fn default() -> MultiCoreSettings {
+        Self::single_threaded()
+    }
+
+    /// Creates multi core settings so that no thread will be spawned.
+    pub fn single_threaded() -> MultiCoreSettings {
+        Self::new(1)
+    }
+
+    /// Creates multi core so that threads will be spawned if this appears to be beneficial.
+    pub fn parallel() -> MultiCoreSettings {
+        if *NUMBER_OF_CORES == 1 {
+            Self::new(1)
+        } else if *NUMBER_OF_CORES == 2 {
+            // If two cores are available then use both of them if multi-threading is requested
+            Self::new(2)
+        } else {
+            // Half because we assume hyper threading and that we will keep a core so busy
+            // that hyper threading isn't of any use
+            Self::new(*NUMBER_OF_CORES / 2)
+        }
+    }
+
+    /// Creates multi core settings with the given values.
+    ///
+    /// If `core_limit > 1` then the first time this function is executed a calibration run
+    /// will be performed which determines the details for the multi threading strategy. This
+    /// run takes between 200ms and 500ms.
+    pub fn new(core_limit: usize) -> MultiCoreSettings {
+        if core_limit <= 1 {
+            MultiCoreSettings {
+                core_limit: 1,
+                med_dual_core_threshold: MED_DUAL_CORE_DEFAULT,
+                med_multi_core_threshold: MED_MULTI_CORE_DEFAULT,
+                large_dual_core_threshold: LARGE_DUAL_CORE_DEFAULT,
+                large_multi_core_threshold: LARGE_MULTI_CORE_DEFAULT,
+            }
+        } else {
+            MultiCoreSettings {
+                core_limit,
+                med_dual_core_threshold: CALIBRATION.med_dual_core_threshold,
+                med_multi_core_threshold: CALIBRATION.med_multi_core_threshold,
+                large_dual_core_threshold: CALIBRATION.large_dual_core_threshold,
+                large_multi_core_threshold: CALIBRATION.large_multi_core_threshold,
+            }
+        }
+    }
+
+    /// Creates multi core settings with explicit thresholds.
+    ///
+    /// The arguments allow to specify the number of cores which are used at maximum and as
+    /// well as starting from which slice size the lib will start to use two or all cores.
+    /// `med` denotes operations of medium complexity, e.g. such as log and sin.
+    /// `large` denotes operations like convolution and interpolation.
+    pub fn with_thresholds(
+        core_limit: usize,
+        med_dual_core_threshold: usize,
+        med_multi_core_threshold: usize,
+        large_dual_core_threshold: usize,
+        large_multi_core_threshold: usize,
+    ) -> MultiCoreSettings {
+        MultiCoreSettings {
+            core_limit,
+            med_dual_core_threshold,
+            med_multi_core_threshold: med_multi_core_threshold.max(med_dual_core_threshold + 1),
+            large_dual_core_threshold,
+            large_multi_core_threshold: large_multi_core_threshold
+                .max(large_dual_core_threshold + 1),
+        }
+    }
 }
 
 /// Prints debug information about the calibration. The calibration determines when the library
@@ -269,7 +316,7 @@ impl Chunk {
     fn determine_number_of_chunks(
         array_length: usize,
         complexity: Complexity,
-        settings: MultiCoreSettings,
+        settings: &MultiCoreSettings,
     ) -> usize {
         if settings.core_limit == 1 {
             settings.core_limit
@@ -279,24 +326,19 @@ impl Chunk {
             // is already fast enough to occupy the max memory bandwidth
             1
         } else if complexity == Complexity::Medium {
-            let calibration = &CALIBRATION;
-            if array_length < calibration.med_dual_core_threshold {
+            if array_length < settings.med_dual_core_threshold {
                 1
-            } else if array_length < calibration.med_multi_core_threshold {
+            } else if array_length < settings.med_multi_core_threshold {
                 2
             } else {
                 settings.core_limit
             }
+        } else if array_length < settings.large_dual_core_threshold {
+            1
+        } else if array_length < settings.large_multi_core_threshold {
+            2
         } else {
-            // Complexity::Large
-            let calibration = &CALIBRATION;
-            if array_length < calibration.large_dual_core_threshold {
-                1
-            } else if array_length < calibration.large_multi_core_threshold {
-                2
-            } else {
-                settings.core_limit
-            }
+            settings.core_limit
         }
     }
 
@@ -367,7 +409,7 @@ impl Chunk {
     #[inline]
     pub fn execute_partial<'a, T, S, F>(
         complexity: Complexity,
-        settings: MultiCoreSettings,
+        settings: &MultiCoreSettings,
         array: &mut [T],
         step_size: usize,
         arguments: S,
@@ -408,96 +450,12 @@ impl Chunk {
         });
     }
 
-    /// Executes the given function on the first `array_length` elements of the given list of
-    /// arrays in parallel and passes the argument to all function calls.
-    #[inline]
-    pub fn execute_partial_multidim<'a, T, S, F>(
-        complexity: Complexity,
-        settings: MultiCoreSettings,
-        array: &mut [&mut [T]],
-        range: Range<usize>,
-        step_size: usize,
-        arguments: S,
-        ref function: F,
-    ) where
-        F: Fn(&mut InlineVector<&mut [T]>, Range<usize>, S) + 'a + Sync,
-        T: RealNumber,
-        S: Sync + Copy + Send,
-    {
-        let dimensions = array.len();
-        let number_of_chunks =
-            Chunk::determine_number_of_chunks(range.end - range.start, complexity, settings);
-        if number_of_chunks > 1 {
-            let ranges =
-                Chunk::partition_in_ranges(range.end - range.start, step_size, number_of_chunks);
-            // As an example: If this function is called with 2 arrays,
-            // and each array has 1000 elements and is split into 4 chunks then
-            // this:
-            // let matrix = array.iter_mut().map(|a| {
-            //    Chunk::partition_mut(a, array_length, step_size, number_of_chunks)
-            // }).collect();
-            // would give us a layout like this:
-            // Vector with 2 elements * 4 chunks * each chunk with 250 elements (mutable)
-            //
-            // But what we want to have is:
-            // 4 chunks * a vector of 2 elements * 250 elements in each vector (mutable)
-            //
-            // We achieve this by a) flatten the structure (flat_map) so that we get:
-            // array1 chunk1
-            // array1 chunk2
-            // array1 chunk3
-            // array1 chunk4
-            // array2 chunk1
-            // array2 chunk2
-            // array2 chunk3
-            // array2 chunk4
-            //
-            // and b) creating a nested vector structure so that we can push
-            // the elements on it in the correct order
-            let mut flat_layout: Vec<&mut [T]> = array
-                .iter_mut()
-                .flat_map(|a| {
-                    Chunk::partition_mut(
-                        &mut a[range.start..range.end],
-                        step_size,
-                        number_of_chunks,
-                    )
-                })
-                .collect();
-
-            let mut reorganized = Vec::with_capacity(number_of_chunks);
-            for _ in 0..number_of_chunks {
-                reorganized.push(InlineVector::with_capacity(dimensions));
-            }
-            let mut i = flat_layout.len();
-            while i > 0 {
-                let elem = flat_layout.pop().unwrap();
-                reorganized[i % number_of_chunks].push(elem);
-                i -= 1;
-            }
-
-            crossbeam::scope(|scope| {
-                for chunk in reorganized.iter_mut().zip(ranges) {
-                    scope.spawn(move || {
-                        function(chunk.0, chunk.1, arguments);
-                    });
-                }
-            });
-        } else {
-            let mut shortened = InlineVector::with_capacity(range.end - range.start);
-            for n in array.iter_mut().map(|a| &mut a[range.start..range.end]) {
-                shortened.push(n);
-            }
-            function(&mut shortened, range, arguments);
-        }
-    }
-
     /// Executes the given function on the all elements of the array and also tells the function
     /// on which range/chunk it operates on.
     #[inline]
     pub fn execute_with_range<'a, T, S, F>(
         complexity: Complexity,
-        settings: MultiCoreSettings,
+        settings: &MultiCoreSettings,
         array: &mut [T],
         step_size: usize,
         arguments: S,
@@ -537,7 +495,7 @@ impl Chunk {
     #[inline]
     pub fn map_on_array_chunks<'a, T, S, F, R>(
         complexity: Complexity,
-        settings: MultiCoreSettings,
+        settings: &MultiCoreSettings,
         array: &[T],
         step_size: usize,
         arguments: S,
@@ -590,7 +548,7 @@ impl Chunk {
     #[inline]
     pub fn execute_sym_pairs_with_range<'a, T, S, F>(
         complexity: Complexity,
-        settings: MultiCoreSettings,
+        settings: &MultiCoreSettings,
         array: &mut [T],
         step_size: usize,
         arguments: S,
@@ -652,9 +610,9 @@ impl Chunk {
     /// Executes the given function on the all elements of the array in parallel. A result is
     /// returned for each chunk.
     #[inline]
-    pub fn get_a_fold_b<'a, F, T, R>(
+    pub fn get_zipped<'a, F, T, R>(
         complexity: Complexity,
-        settings: MultiCoreSettings,
+        settings: &MultiCoreSettings,
         a: &[T],
         a_step: usize,
         b: &[T],
@@ -666,6 +624,10 @@ impl Chunk {
         T: Float + Copy + Clone + Send + Sync,
         R: Send,
     {
+        if a.is_empty() {
+            return InlineVector::empty();
+        }
+
         let a_len = a.len();
         let b_len = b.len();
         let number_of_chunks = Chunk::determine_number_of_chunks(a_len, complexity, settings);
@@ -703,7 +665,7 @@ impl Chunk {
     #[inline]
     pub fn get_chunked_results<'a, F, S, T, R>(
         complexity: Complexity,
-        settings: MultiCoreSettings,
+        settings: &MultiCoreSettings,
         a: &[T],
         a_step: usize,
         arguments: S,
@@ -715,6 +677,10 @@ impl Chunk {
         R: Send,
         S: Sync + Copy + Send,
     {
+        if a.is_empty() {
+            return InlineVector::empty();
+        }
+
         let a_len = a.len();
         let number_of_chunks = Chunk::determine_number_of_chunks(a_len, complexity, settings);
         if number_of_chunks > 1 {
@@ -752,7 +718,7 @@ impl Chunk {
     #[inline]
     pub fn from_src_to_dest<'a, T, S, F>(
         complexity: Complexity,
-        settings: MultiCoreSettings,
+        settings: &MultiCoreSettings,
         original: &[T],
         original_step: usize,
         target: &mut [T],
