@@ -26,26 +26,20 @@ impl f32x4 {
 
     #[inline]
     pub fn load(array: &[f32], index: usize) -> Self {
-        let array = &array[index..index + 4];
-        unsafe {
-            f32x4(
-                *array.get_unchecked(0),
-                *array.get_unchecked(1),
-                *array.get_unchecked(2),
-                *array.get_unchecked(3),
-            )
-        }
+        f32x4(
+            array[index],
+            array[index + 1],
+            array[index + 2],
+            array[index + 3],
+        )
     }
 
     #[inline]
     pub fn store(self, array: &mut [f32], index: usize) {
-        let array = &mut array[index..index + 4];
-        unsafe {
-            *array.get_unchecked_mut(0) = self.0;
-            *array.get_unchecked_mut(1) = self.1;
-            *array.get_unchecked_mut(2) = self.2;
-            *array.get_unchecked_mut(3) = self.3;
-        }
+        array[index] = self.extract(0);
+        array[index + 1] = self.extract(1);
+        array[index + 2] = self.extract(2);
+        array[index + 3] = self.extract(3);
     }
 
     #[inline]
@@ -73,17 +67,13 @@ impl f64x2 {
 
     #[inline]
     pub fn load(array: &[f64], index: usize) -> Self {
-        let array = &array[index..index + 2];
-        unsafe { f64x2(*array.get_unchecked(0), *array.get_unchecked(1)) }
+        f64x2(array[index], array[index + 1])
     }
 
     #[inline]
     pub fn store(self, array: &mut [f64], index: usize) {
-        let array = &mut array[index..index + 2];
-        unsafe {
-            *array.get_unchecked_mut(0) = self.0;
-            *array.get_unchecked_mut(1) = self.1;
-        }
+        array[index] = self.extract(0);
+        array[index + 1] = self.extract(1);
     }
 
     #[inline]
@@ -174,15 +164,6 @@ impl Simd<f32> for f32x4 {
 
     const LEN: usize = 4;
 
-    #[inline]
-    fn load_wrap_unchecked(array: &[f32], idx: usize) -> f32x4 {
-        let mut temp = [0.0; 4];
-        for (i, t) in temp.iter_mut().enumerate() {
-            *t = unsafe { *array.get_unchecked((idx + i) % array.len()) };
-        }
-        f32x4::load(&temp, 0)
-    }
-
     #[inline]
     fn from_complex(value: Complex<f32>) -> f32x4 {
         f32x4::new(value.re, value.im, value.re, value.im)
@@ -253,23 +234,6 @@ impl Simd<f32> for f32x4 {
             0.0,
         )
     }
-    #[inline]
-
-    fn complex_abs_squared2(self) -> f32x4 {
-        let squared = self * self;
-        f32x4::new(squared.0 + squared.1, 0.0, squared.2 + squared.3, 0.0)
-    }
-
-    #[inline]
-    fn complex_abs2(self) -> f32x4 {
-        let squared = self * self;
-        f32x4::new(
-            (squared.0 + squared.1).sqrt(),
-            0.0,
-            (squared.2 + squared.3).sqrt(),
-            0.0,
-        )
-    }
 
     #[inline]
     fn sqrt(self) -> f32x4 {
@@ -277,11 +241,9 @@ impl Simd<f32> for f32x4 {
     }
 
     #[inline]
-    fn store_half_unchecked(self, target: &mut [f32], index: usize) {
-        unsafe {
-            *target.get_unchecked_mut(index) = self.0;
-            *target.get_unchecked_mut(index + 1) = self.1;
-        }
+    fn store_half(self, target: &mut [f32], index: usize) {
+        target[index] = self.extract(0);
+        target[index + 1] = self.extract(1);
     }
 
     #[inline]
@@ -339,16 +301,6 @@ impl Simd<f64> for f64x2 {
 
     const LEN: usize = 2;
 
-    #[inline]
-    fn load_wrap_unchecked(array: &[f64], idx: usize) -> f64x2 {
-        let mut temp = [0.0; 2];
-        for (i, t) in temp.iter_mut().enumerate() {
-            *t = unsafe { *array.get_unchecked((idx + i) % array.len()) };
-        }
-
-        f64x2::load(&temp, 0)
-    }
-
     #[inline]
     fn from_complex(value: Complex<f64>) -> f64x2 {
         f64x2::new(value.re, value.im)
@@ -411,32 +363,14 @@ impl Simd<f64> for f64x2 {
         f64x2::new(result, 0.0)
     }
 
-    #[inline]
-    fn complex_abs_squared2(self) -> f64x2 {
-        let a = self.0;
-        let b = self.1;
-        let result = a * a + b * b;
-        f64x2::new(result, 0.0)
-    }
-
-    #[inline]
-    fn complex_abs2(self) -> f64x2 {
-        let a = self.0;
-        let b = self.1;
-        let result = (a * a + b * b).sqrt();
-        f64x2::new(result, 0.0)
-    }
-
     #[inline]
     fn sqrt(self) -> f64x2 {
         f64x2::new(self.0.sqrt(), self.1.sqrt())
     }
 
     #[inline]
-    fn store_half_unchecked(self, target: &mut [f64], index: usize) {
-        unsafe {
-            *target.get_unchecked_mut(index) = self.0;
-        }
+    fn store_half(self, target: &mut [f64], index: usize) {
+        target[index] = self.extract(0);
     }
 
     #[inline]
