@@ -1,5 +1,6 @@
 use super::{Simd, SimdFrom};
 use numbers::*;
+use simd;
 use simd::x86::sse2::*;
 use simd::*;
 use std::arch::x86_64::*;
@@ -24,15 +25,6 @@ impl Simd<f32> for f32x4 {
 
     const LEN: usize = 4;
 
-    #[inline]
-    fn load_wrap_unchecked(array: &[f32], idx: usize) -> f32x4 {
-        let mut temp = [0.0; 4];
-        for (i, t) in temp.iter_mut().enumerate() {
-            *t = unsafe { *array.get_unchecked((idx + i) % array.len()) };
-        }
-        f32x4::load(&temp, 0)
-    }
-
     #[inline]
     fn from_complex(value: Complex<f32>) -> f32x4 {
         f32x4::new(value.re, value.im, value.re, value.im)
@@ -147,39 +139,15 @@ impl Simd<f32> for f32x4 {
         squared_sum.sqrt()
     }
 
-    #[inline]
-    fn complex_abs_squared2(self) -> f32x4 {
-        let abs = self.complex_abs_squared();
-        f32x4::new(
-            abs.extract(0),
-            abs.extract(2),
-            abs.extract(1),
-            abs.extract(3),
-        )
-    }
-
-    #[inline]
-    fn complex_abs2(self) -> f32x4 {
-        let abs = self.complex_abs();
-        f32x4::new(
-            abs.extract(0),
-            abs.extract(2),
-            abs.extract(1),
-            abs.extract(3),
-        )
-    }
-
     #[inline]
     fn sqrt(self) -> f32x4 {
         self.sqrt()
     }
 
     #[inline]
-    fn store_half_unchecked(self, target: &mut [f32], index: usize) {
-        unsafe {
-            *target.get_unchecked_mut(index) = self.extract(0);
-            *target.get_unchecked_mut(index + 1) = self.extract(1);
-        }
+    fn store_half(self, target: &mut [f32], index: usize) {
+        target[index] = self.extract(0);
+        target[index + 1] = self.extract(1);
     }
 
     #[inline]
@@ -225,16 +193,6 @@ impl Simd<f64> for f64x2 {
 
     const LEN: usize = 2;
 
-    #[inline]
-    fn load_wrap_unchecked(array: &[f64], idx: usize) -> f64x2 {
-        let mut temp = [0.0; 2];
-        for (i, t) in temp.iter_mut().enumerate() {
-            *t = unsafe { *array.get_unchecked((idx + i) % array.len()) };
-        }
-
-        f64x2::load(&temp, 0)
-    }
-
     #[inline]
     fn from_complex(value: Complex<f64>) -> f64x2 {
         f64x2::new(value.re, value.im)
@@ -297,26 +255,14 @@ impl Simd<f64> for f64x2 {
         f64x2::new(result, 0.0)
     }
 
-    #[inline]
-    fn complex_abs_squared2(self) -> f64x2 {
-        self.complex_abs_squared()
-    }
-
-    #[inline]
-    fn complex_abs2(self) -> f64x2 {
-        self.complex_abs()
-    }
-
     #[inline]
     fn sqrt(self) -> f64x2 {
         simd::x86::sse2::Sse2F64x2::sqrt(self)
     }
 
     #[inline]
-    fn store_half_unchecked(self, target: &mut [f64], index: usize) {
-        unsafe {
-            *target.get_unchecked_mut(index) = self.extract(0);
-        }
+    fn store_half(self, target: &mut [f64], index: usize) {
+        target[index] = self.extract(0);
     }
 
     #[inline]
