@@ -1,8 +1,8 @@
 //! Defines the buffers. Finding a good general purpose buffering scheme is hard.
 //! So the best option seems to be to create an abstraction so that the buffering can be adjusted
 //! to what an application needs.
-use numbers::*;
 use super::ToSliceMut;
+use numbers::*;
 use std::ops::*;
 
 /// A "slice-like" type which also allows to
@@ -15,8 +15,9 @@ pub trait BufferBorrow<S: ToSliceMut<T>, T: RealNumber>: DerefMut<Target = [T]>
 /// A buffer which can be used by other types. Types will call buffers to create new arrays.
 /// A buffer may can implement any buffering strategy.
 pub trait Buffer<'a, S, T>
-    where S: ToSliceMut<T>,
-          T: RealNumber
+where
+    S: ToSliceMut<T>,
+    T: RealNumber,
 {
     /// The type of the burrow which is returned.
     type Borrow: BufferBorrow<S, T>;
