@@ -64,7 +64,7 @@ macro_rules! assert_complex {
     ($self_: ident) => {
         if !$self_.is_complex() {
             $self_.number_space.to_real();
-            $self_.valid_len = 0;
+            $self_.mark_vector_as_invalid();
         }
     };
 }
@@ -84,7 +84,7 @@ where
         let array = self.data.to_slice_mut();
         Chunk::execute_with_range(
             Complexity::Small,
-            self.multicore_settings,
+            &self.multicore_settings,
             &mut array[0..data_length],
             2,
             (a, b),
