@@ -3,6 +3,7 @@ use super::super::{
     PosEq, RealNumberSpace, RededicateForceOps, Resize, ToRealResult, ToSlice, ToSliceMut, Vector,
     VoidResult,
 };
+use array_to_complex;
 use multicore_support::*;
 use numbers::*;
 use simd_extensions::*;
@@ -340,7 +341,7 @@ macro_rules! assert_complex {
     ($self_: ident) => {
         if !$self_.is_complex() {
             $self_.number_space.to_real();
-            $self_.valid_len = 0;
+            $self_.mark_vector_as_invalid();
             return Self::RealResult::rededicate_from_force($self_);
         }
     };
@@ -489,7 +490,7 @@ where
         let source = &self.data.to_slice();
         Chunk::from_src_to_dest(
             complexity,
-            self.multicore_settings,
+            &self.multicore_settings,
             &source[0..len],
             2,
             array,
@@ -528,42 +529,31 @@ where
         destination.set_delta(self.delta);
         let temp = &mut destination[0..data_length / 2];
         let array = &self.data.to_slice();
-        let (scalar_left, scalar_right, vectorization_length) =
-            Reg::calc_data_alignment_reqs(array);
-        if vectorization_length.is_some() {
-            let vectorization_length = vectorization_length.unwrap();
-            Chunk::from_src_to_dest(
-                complexity,
-                self.multicore_settings,
-                &array[scalar_left..vectorization_length],
-                Reg::LEN,
-                &mut temp[scalar_left / 2..vectorization_length / 2],
-                Reg::LEN / 2,
-                (),
-                move |array, range, target, _arg| {
-                    let mut i = 0;
-                    let array = Reg::array_to_regs(&array[range]);
-                    for n in array {
-                        let result = op_simd(*n);
-                        result.store_half_unchecked(target, i);
-                        i += Reg::LEN / 2;
-                    }
-                },
-            );
-        }
+        let partition = Reg::calc_data_alignment_reqs(array);
+        Chunk::from_src_to_dest(
+            complexity,
+            &self.multicore_settings,
+            partition.center(array),
+            Reg::LEN,
+            partition.rcenter_mut(temp),
+            Reg::LEN / 2,
+            (),
+            move |array, range, target, _arg| {
+                let mut i = 0;
+                let array = Reg::array_to_regs(&array[range]);
+                for n in array {
+                    let result = op_simd(*n);
+                    result.store_half(target, i);
+                    i += Reg::LEN / 2;
+                }
+            },
+        );
 
-        let mut i = 0;
-        while i < scalar_left {
-            let c = Complex::new(array[i], array[i + 1]);
-            temp[i / 2] = op(c);
-            i += 2;
-        }
-
-        let mut i = scalar_right;
-        while i < data_length {
-            let c = Complex::new(array[i], array[i + 1]);
-            temp[i / 2] = op(c);
-            i += 2;
+        for (dest, src) in partition
+            .redge_iter_mut(temp)
+            .zip(partition.cedge_iter(array_to_complex(array)))
+        {
+            *dest = op(*src);
         }
     }
 }
