@@ -263,7 +263,7 @@ macro_rules! zero_interleave {
             let source = &$self_.data.to_slice();
             Chunk::from_src_to_dest(
                 Complexity::Small,
-                $self_.multicore_settings,
+                &$self_.multicore_settings,
                 &source[0..old_len],
                 $tuple,
                 &mut target[0..new_len],
@@ -279,8 +279,8 @@ macro_rules! zero_interleave {
                     let mut i = 0;
                     let mut j = range.start;
                     while i < target.len() {
-                        let original_ptr = unsafe { original.get_unchecked(j) };
-                        let target_ptr = unsafe { target.get_unchecked_mut(i) };
+                        let original_ptr = &original[j];
+                        let target_ptr = &mut target[i];
                         unsafe {
                             ptr::copy(original_ptr, target_ptr, $tuple);
                         }
