@@ -3,7 +3,7 @@ use super::super::{
     RealNumberSpace, ScalarResult, ToSlice, Vector,
 };
 use super::kahan_sum;
-use inline_vector::InlineVector;
+use array_to_complex;
 use multicore_support::*;
 use numbers::*;
 use simd_extensions::*;
@@ -81,46 +81,32 @@ where
 
         let data_length = self.len();
         let array = self.data.to_slice();
-        let (scalar_left, scalar_right, vectorization_length) =
-            Reg::calc_data_alignment_reqs(&array[0..data_length]);
+        let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
         let other = &factor[..];
-        let chunks = if vectorization_length.is_some() {
-            let vectorization_length = vectorization_length.unwrap();
-            Chunk::get_a_fold_b(
-                Complexity::Small,
-                self.multicore_settings,
-                &other[scalar_left..vectorization_length],
-                Reg::LEN,
-                &array[scalar_left..vectorization_length],
-                Reg::LEN,
-                |original, range, target| {
-                    let mut result = Reg::splat(T::zero());
-                    let mut i = range.start;
-                    let target = Reg::array_to_regs(&target[..]);
-                    for a in target {
-                        result = result + *a * Reg::load_unchecked(original, i);
-                        i += Reg::LEN;
-                    }
+        let chunks = Chunk::get_zipped(
+            Complexity::Small,
+            &self.multicore_settings,
+            partition.center(other),
+            Reg::LEN,
+            partition.center(array),
+            Reg::LEN,
+            |original, range, target| {
+                let mut result = Reg::splat(T::zero());
+                let mut i = range.start;
+                let target = Reg::array_to_regs(&target[..]);
+                for a in target {
+                    result = result + *a * Reg::load(original, i);
+                    i += Reg::LEN;
+                }
 
-                    result.sum_real()
-                },
-            )
-        } else {
-            InlineVector::empty()
-        };
+                result.sum_real()
+            },
+        );
 
-        let mut i = 0;
-        let mut sum = T::zero();
-        while i < scalar_left {
-            sum = sum + array[i] * other[i];
-            i += 1;
-        }
-
-        let mut i = scalar_right;
-        while i < data_length {
-            sum = sum + array[i] * other[i];
-            i += 1;
-        }
+        let sum = partition
+            .edge_iter(array)
+            .zip(partition.edge_iter(other))
+            .fold(T::zero(), |sum, (a, b)| sum + *a * *b);
 
         let chunk_sum: T = (&chunks[..]).iter().fold(T::zero(), |a, b| a + *b);
         Ok(chunk_sum + sum)
@@ -146,49 +132,31 @@ where
 
         let data_length = self.len();
         let array = self.data.to_slice();
-        let (scalar_left, scalar_right, vectorization_length) =
-            Reg::calc_data_alignment_reqs(&array[0..data_length]);
+        let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
         let other = &factor[..];
-        let chunks = if vectorization_length.is_some() {
-            let vectorization_length = vectorization_length.unwrap();
-            Chunk::get_a_fold_b(
-                Complexity::Small,
-                self.multicore_settings,
-                &other[scalar_left..vectorization_length],
-                Reg::LEN,
-                &array[scalar_left..vectorization_length],
-                Reg::LEN,
-                |original, range, target| {
-                    let mut result = Reg::splat(T::zero());
-                    let mut i = range.start;
-                    let target = Reg::array_to_regs(&target[..]);
-                    for a in target {
-                        result = result + a.mul_complex(Reg::load_unchecked(original, i));
-                        i += Reg::LEN;
-                    }
-                    result.sum_complex()
-                },
-            )
-        } else {
-            InlineVector::empty()
-        };
+        let chunks = Chunk::get_zipped(
+            Complexity::Small,
+            &self.multicore_settings,
+            partition.center(other),
+            Reg::LEN,
+            partition.center(array),
+            Reg::LEN,
+            |original, range, target| {
+                let mut result = Reg::splat(T::zero());
+                let mut i = range.start;
+                let target = Reg::array_to_regs(&target[..]);
+                for a in target {
+                    result = result + a.mul_complex(Reg::load(original, i));
+                    i += Reg::LEN;
+                }
+                result.sum_complex()
+            },
+        );
 
-        let mut i = 0;
-        let mut sum = Complex::<T>::new(T::zero(), T::zero());
-        while i < scalar_left {
-            let a = Complex::<T>::new(array[i], array[i + 1]);
-            let b = Complex::<T>::new(other[i], other[i + 1]);
-            sum = sum + a * b;
-            i += 2;
-        }
-
-        let mut i = scalar_right;
-        while i < data_length {
-            let a = Complex::<T>::new(array[i], array[i + 1]);
-            let b = Complex::<T>::new(other[i], other[i + 1]);
-            sum = sum + a * b;
-            i += 2;
-        }
+        let sum = partition
+            .cedge_iter(array_to_complex(array))
+            .zip(partition.cedge_iter(array_to_complex(other)))
+            .fold(Complex::<T>::zero(), |sum, (a, b)| sum + *a * *b);
 
         let chunk_sum: Complex<T> = (&chunks[..])
             .iter()
@@ -212,45 +180,31 @@ where
 
         let data_length = self.len();
         let array = self.data.to_slice();
-        let (scalar_left, scalar_right, vectorization_length) =
-            Reg::calc_data_alignment_reqs(&array[0..data_length]);
+        let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
         let other = &factor[..];
-        let chunks = if vectorization_length.is_some() {
-            let vectorization_length = vectorization_length.unwrap();
-            Chunk::get_a_fold_b(
-                Complexity::Small,
-                self.multicore_settings,
-                &other[scalar_left..vectorization_length],
-                Reg::LEN,
-                &array[scalar_left..vectorization_length],
-                Reg::LEN,
-                |original, range, target| {
-                    let mut i = range.start;
-                    let target = Reg::array_to_regs(&target[..]);
-                    kahan_sum(target.iter().map(|a| {
-                        let res = *a * Reg::load_unchecked(original, i);
-                        i += Reg::LEN;
-                        res
-                    }))
-                    .sum_real()
-                },
-            )
-        } else {
-            InlineVector::empty()
-        };
+        let chunks = Chunk::get_zipped(
+            Complexity::Small,
+            &self.multicore_settings,
+            partition.center(&other),
+            Reg::LEN,
+            partition.center(array),
+            Reg::LEN,
+            |original, range, target| {
+                let mut i = range.start;
+                let target = Reg::array_to_regs(&target[..]);
+                kahan_sum(target.iter().map(|a| {
+                    let res = *a * Reg::load(original, i);
+                    i += Reg::LEN;
+                    res
+                }))
+                .sum_real()
+            },
+        );
 
-        let mut i = 0;
-        let mut sum = T::zero();
-        while i < scalar_left {
-            sum = sum + array[i] * other[i];
-            i += 1;
-        }
-
-        let mut i = scalar_right;
-        while i < data_length {
-            sum = sum + array[i] * other[i];
-            i += 1;
-        }
+        let sum = partition
+            .edge_iter(array)
+            .zip(partition.edge_iter(other))
+            .fold(T::zero(), |sum, (a, b)| sum + *a * *b);
 
         let chunk_sum: T = (&chunks[..]).iter().fold(T::zero(), |a, b| a + *b);
         Ok(chunk_sum + sum)
@@ -276,49 +230,31 @@ where
 
         let data_length = self.len();
         let array = self.data.to_slice();
-        let (scalar_left, scalar_right, vectorization_length) =
-            Reg::calc_data_alignment_reqs(&array[0..data_length]);
+        let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
         let other = &factor[..];
-        let chunks = if vectorization_length.is_some() {
-            let vectorization_length = vectorization_length.unwrap();
-            Chunk::get_a_fold_b(
-                Complexity::Small,
-                self.multicore_settings,
-                &other[scalar_left..vectorization_length],
-                Reg::LEN,
-                &array[scalar_left..vectorization_length],
-                Reg::LEN,
-                |original, range, target| {
-                    let mut i = range.start;
-                    let target = Reg::array_to_regs(&target[..]);
-                    kahan_sum(target.iter().map(|a| {
-                        let res = a.mul_complex(Reg::load_unchecked(original, i));
-                        i += Reg::LEN;
-                        res
-                    }))
-                    .sum_complex()
-                },
-            )
-        } else {
-            InlineVector::empty()
-        };
+        let chunks = Chunk::get_zipped(
+            Complexity::Small,
+            &self.multicore_settings,
+            partition.center(other),
+            Reg::LEN,
+            partition.center(array),
+            Reg::LEN,
+            |original, range, target| {
+                let mut i = range.start;
+                let target = Reg::array_to_regs(&target[..]);
+                kahan_sum(target.iter().map(|a| {
+                    let res = a.mul_complex(Reg::load(original, i));
+                    i += Reg::LEN;
+                    res
+                }))
+                .sum_complex()
+            },
+        );
 
-        let mut i = 0;
-        let mut sum = Complex::<T>::new(T::zero(), T::zero());
-        while i < scalar_left {
-            let a = Complex::<T>::new(array[i], array[i + 1]);
-            let b = Complex::<T>::new(other[i], other[i + 1]);
-            sum = sum + a * b;
-            i += 2;
-        }
-
-        let mut i = scalar_right;
-        while i < data_length {
-            let a = Complex::<T>::new(array[i], array[i + 1]);
-            let b = Complex::<T>::new(other[i], other[i + 1]);
-            sum = sum + a * b;
-            i += 2;
-        }
+        let sum = partition
+            .cedge_iter(array_to_complex(array))
+            .zip(partition.cedge_iter(array_to_complex(other)))
+            .fold(Complex::<T>::zero(), |sum, (a, b)| sum + *a * *b);
 
         let chunk_sum: Complex<T> = (&chunks[..])
             .iter()
