@@ -274,7 +274,7 @@ where
 macro_rules! assert_complex {
     ($self_: ident) => {
         if !$self_.is_complex() {
-            $self_.valid_len = 0;
+            $self_.mark_vector_as_invalid();
             return;
         }
     };
@@ -391,30 +391,24 @@ macro_rules! impl_binary_vector_operation {
 
                 let data_length = self.len();
                 let array = self.data.to_slice_mut();
-                let (scalar_left, scalar_right, vectorization_length) =
-                    Reg::calc_data_alignment_reqs(&array[0..data_length]);
+                let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
                 let other = &$arg_name[..];
-                if vectorization_length.is_some() {
-                    let vectorization_length = vectorization_length.unwrap();
-                    Chunk::from_src_to_dest(
-                        Complexity::Small, self.multicore_settings,
-                        &other[scalar_left..vectorization_length], Reg::LEN,
-                        &mut array[scalar_left..vectorization_length], Reg::LEN, (),
-                        |original, range, target, _arg| {
-                            let mut i = range.start;
-                            let target =
-                                Reg::array_to_regs_mut(&mut target[..]);
-                            for dst in &mut target[..] {
-                                 *dst = dst.$simd_op(Reg::load_unchecked(original, i));
-                                i += Reg::LEN;
-                            }
-                    });
-                }
-                for i in 0..scalar_left {
-                    array[i] = array[i].$scal_op(other[i]);
-                }
-                for i in scalar_right..data_length {
-                    array[i] = array[i].$scal_op(other[i]);
+                Chunk::from_src_to_dest(
+                    Complexity::Small, &self.multicore_settings,
+                    partition.center(other), Reg::LEN,
+                    partition.center_mut(array), Reg::LEN, (),
+                    |original, range, target, _arg| {
+                        let mut i = range.start;
+                        let target =
+                            Reg::array_to_regs_mut(&mut target[..]);
+                        for dst in &mut target[..] {
+                             *dst = dst.$simd_op(Reg::load(original, i));
+                            i += Reg::LEN;
+                        }
+                });
+
+                for (n, o) in partition.edge_iter_mut(array).zip(partition.edge_iter(other)) {
+                    *n = n.$scal_op(*o);
                 }
             }
 
@@ -434,43 +428,24 @@ macro_rules! impl_binary_complex_vector_operation {
 
                 let data_length = self.len();
                 let array = self.data.to_slice_mut();
-                let (scalar_left, scalar_right, vectorization_length) =
-                    Reg::calc_data_alignment_reqs(&array[0..data_length]);
+                let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
                 let other = &$arg_name[..];
-                if vectorization_length.is_some() {
-                    let vectorization_length = vectorization_length.unwrap();
-                    Chunk::from_src_to_dest(
-                        Complexity::Small, self.multicore_settings,
-                        &other[scalar_left..vectorization_length], Reg::LEN,
-                        &mut array[scalar_left..vectorization_length], Reg::LEN, (),
-                        |original, range, target, _arg| {
-                            let mut i = range.start;
-                            let target =
-                                Reg::array_to_regs_mut(&mut target[..]);
-                            for dst in &mut target[..] {
-                                 *dst = dst.$simd_op(Reg::load_unchecked(original, i));
-                                i += Reg::LEN;
-                            }
-                    });
-                }
-                let mut i = 0;
-                while i < scalar_left {
-                    let complex1 = Complex::<T>::new(array[i], array[i + 1]);
-                    let complex2 = Complex::<T>::new(other[i], other[i + 1]);
-                    let result = complex1.$scal_op(complex2);
-                    array[i] = result.re;
-                    array[i + 1] = result.im;
-                    i += 2;
-                }
+                Chunk::from_src_to_dest(
+                    Complexity::Small, &self.multicore_settings,
+                    partition.center(other), Reg::LEN,
+                    partition.center_mut(array), Reg::LEN, (),
+                    |original, range, target, _arg| {
+                        let mut i = range.start;
+                        let target =
+                            Reg::array_to_regs_mut(&mut target[..]);
+                        for dst in &mut target[..] {
+                             *dst = dst.$simd_op(Reg::load(original, i));
+                            i += Reg::LEN;
+                        }
+                });
 
-                let mut i = scalar_right;
-                while i < data_length {
-                    let complex1 = Complex::<T>::new(array[i], array[i + 1]);
-                    let complex2 = Complex::<T>::new(other[i], other[i + 1]);
-                    let result = complex1.$scal_op(complex2);
-                    array[i] = result.re;
-                    array[i + 1] = result.im;
-                    i += 2;
+                for (n, o) in partition.cedge_iter_mut(array_to_complex_mut(array)).zip(partition.cedge_iter(array_to_complex(other))) {
+                    *n = n.$scal_op(*o);
                 }
             }
 
@@ -492,7 +467,7 @@ macro_rules! impl_binary_smaller_vector_operation {
                 let array = self.data.to_slice_mut();
                 let other = &$arg_name[..];
                 Chunk::from_src_to_dest(
-                    Complexity::Small, self.multicore_settings,
+                    Complexity::Small, &self.multicore_settings,
                     &other, Reg::LEN,
                     &mut array[0..data_length], 1, (),
                     |operand, range, target, _arg| {
@@ -522,7 +497,7 @@ macro_rules! impl_binary_smaller_complex_vector_ops {
                 let array = self.data.to_slice_mut();
                 let other = &$arg_name[..];
                 Chunk::from_src_to_dest(
-                    Complexity::Small, self.multicore_settings,
+                    Complexity::Small, &self.multicore_settings,
                     &other, Reg::LEN,
                     &mut array[0..data_length], 2, (),
                     |operand, range, target, _arg| {
