@@ -152,7 +152,7 @@ macro_rules! impl_common_stats {
             if stats.is_empty() {
                 return StatsVec::new();
             }
-        
+
             let len = stats[0].len();
             let mut results = StatsVec::new();
             for i in 0..len {
@@ -160,13 +160,13 @@ macro_rules! impl_common_stats {
                 for s in stats.iter() {
                     reordered.push(s[i]);
                 }
-        
+
                 let merged = Statistics::merge(&reordered);
                 results.push(merged);
             }
             results
         }
-        
+
         fn empty_vec(len: usize) -> StatsVec<Self> {
             let mut results = StatsVec::new();
             for _ in 0..len {
@@ -366,7 +366,7 @@ where
         let array = self.data.to_slice();
         let chunks = Chunk::get_chunked_results(
             Complexity::Small,
-            self.multicore_settings,
+            &self.multicore_settings,
             &array[0..data_length],
             1,
             (),
@@ -407,7 +407,7 @@ where
         let array = self.data.to_slice();
         let chunks = Chunk::get_chunked_results(
             Complexity::Small,
-            self.multicore_settings,
+            &self.multicore_settings,
             &array[0..data_length],
             1,
             len,
@@ -438,14 +438,12 @@ where
     fn sum_real<Reg: SimdGeneric<T>>(&self, _: RegType<Reg>) -> T {
         let data_length = self.len();
         let array = self.data.to_slice();
-        let (scalar_left, scalar_right, vectorization_length) =
-            Reg::calc_data_alignment_reqs(&array[0..data_length]);
-        let mut sum = if vectorization_length.is_some() {
-            let vectorization_length = vectorization_length.unwrap();
+        let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
+        let sum = {
             let chunks = Chunk::get_chunked_results(
                 Complexity::Small,
-                self.multicore_settings,
-                &array[scalar_left..vectorization_length],
+                &self.multicore_settings,
+                partition.center(array),
                 Reg::LEN,
                 (),
                 move |array, _, _| {
@@ -461,29 +459,22 @@ where
                 .iter()
                 .map(|v| v.sum_real())
                 .fold(T::zero(), |a, b| a + b)
-        } else {
-            T::zero()
         };
-        for num in &array[0..scalar_left] {
-            sum = sum + *num;
-        }
-        for num in &array[scalar_right..data_length] {
-            sum = sum + *num;
-        }
-        sum
+
+        sum + partition
+            .edge_iter(array)
+            .fold(T::zero(), |sum, x| sum + *x)
     }
 
     fn sum_sq_real<Reg: SimdGeneric<T>>(&self, _: RegType<Reg>) -> T {
         let data_length = self.len();
         let array = self.data.to_slice();
-        let (scalar_left, scalar_right, vectorization_length) =
-            Reg::calc_data_alignment_reqs(&array[0..data_length]);
-        let mut sum = if vectorization_length.is_some() {
-            let vectorization_length = vectorization_length.unwrap();
+        let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
+        let sum = {
             let chunks = Chunk::get_chunked_results(
                 Complexity::Small,
-                self.multicore_settings,
-                &array[scalar_left..vectorization_length],
+                &self.multicore_settings,
+                partition.center(array),
                 Reg::LEN,
                 (),
                 move |array, _, _| {
@@ -499,29 +490,22 @@ where
                 .iter()
                 .map(|v| v.sum_real())
                 .fold(T::zero(), |a, b| a + b)
-        } else {
-            T::zero()
         };
-        for num in &array[0..scalar_left] {
-            sum = sum + *num * *num;
-        }
-        for num in &array[scalar_right..data_length] {
-            sum = sum + *num * *num;
-        }
-        sum
+
+        sum + partition
+            .edge_iter(array)
+            .fold(T::zero(), |sum, x| sum + *x * *x)
     }
 
     fn sum_complex<Reg: SimdGeneric<T>>(&self, _: RegType<Reg>) -> Complex<T> {
         let data_length = self.len();
         let array = self.data.to_slice();
-        let (scalar_left, scalar_right, vectorization_length) =
-            Reg::calc_data_alignment_reqs(&array[0..data_length]);
-        let mut sum = if vectorization_length.is_some() {
-            let vectorization_length = vectorization_length.unwrap();
+        let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
+        let sum = {
             let chunks = Chunk::get_chunked_results(
                 Complexity::Small,
-                self.multicore_settings,
-                &array[scalar_left..vectorization_length],
+                &self.multicore_settings,
+                partition.center(array),
                 Reg::LEN,
                 (),
                 move |array, _, _| {
@@ -537,29 +521,22 @@ where
                 .iter()
                 .map(|v| v.sum_complex())
                 .fold(Complex::<T>::new(T::zero(), T::zero()), |acc, x| acc + x)
-        } else {
-            Complex::<T>::new(T::zero(), T::zero())
         };
-        for num in array_to_complex(&array[0..scalar_left]) {
-            sum = sum + *num;
-        }
-        for num in array_to_complex(&array[scalar_right..data_length]) {
-            sum = sum + *num;
-        }
-        sum
+
+        sum + partition
+            .cedge_iter(array_to_complex(array))
+            .fold(Complex::<T>::zero(), |sum, x| sum + *x)
     }
 
     fn sum_sq_complex<Reg: SimdGeneric<T>>(&self, _: RegType<Reg>) -> Complex<T> {
         let data_length = self.len();
         let array = self.data.to_slice();
-        let (scalar_left, scalar_right, vectorization_length) =
-            Reg::calc_data_alignment_reqs(&array[0..data_length]);
-        let mut sum = if vectorization_length.is_some() {
-            let vectorization_length = vectorization_length.unwrap();
+        let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
+        let sum = {
             let chunks = Chunk::get_chunked_results(
                 Complexity::Small,
-                self.multicore_settings,
-                &array[scalar_left..vectorization_length],
+                &self.multicore_settings,
+                partition.center(array),
                 Reg::LEN,
                 (),
                 move |array, _, _| {
@@ -575,16 +552,11 @@ where
                 .iter()
                 .map(|v| v.sum_complex())
                 .fold(Complex::<T>::new(T::zero(), T::zero()), |acc, x| acc + x)
-        } else {
-            Complex::<T>::new(T::zero(), T::zero())
         };
-        for num in array_to_complex(&array[0..scalar_left]) {
-            sum = sum + *num * *num;
-        }
-        for num in array_to_complex(&array[scalar_right..data_length]) {
-            sum = sum + *num * *num;
-        }
-        sum
+
+        sum + partition
+            .cedge_iter(array_to_complex(array))
+            .fold(Complex::<T>::zero(), |sum, x| sum + *x * *x)
     }
 }
 
@@ -618,7 +590,7 @@ where
         let array = self.data.to_slice();
         let chunks = Chunk::get_chunked_results(
             Complexity::Small,
-            self.multicore_settings,
+            &self.multicore_settings,
             &array[0..data_length],
             2,
             (),
@@ -660,7 +632,7 @@ where
         let array = self.data.to_slice();
         let chunks = Chunk::get_chunked_results(
             Complexity::Small,
-            self.multicore_settings,
+            &self.multicore_settings,
             &array[0..data_length],
             2,
             len,
