@@ -31,7 +31,6 @@ mod real;
 pub use self::real::*;
 mod time_freq;
 pub use self::time_freq::*;
-pub mod combined_ops;
 mod rededicate_and_relations;
 pub use self::rededicate_and_relations::*;
 mod checks_and_results;
@@ -40,6 +39,7 @@ mod general;
 pub use self::general::*;
 mod buffer;
 pub use self::buffer::*;
+use super::meta;
 
 /// Result for operations which transform a type (most commonly the type is a vector).
 /// On success the transformed type is returned.
@@ -99,114 +99,8 @@ pub trait TimeDomain: Domain {}
 /// Trait for types containing frequency domain data.
 pub trait FrequencyDomain: Domain {}
 
-/// Marker for types containing real data.
-#[derive(Debug, Clone, PartialEq)]
-pub struct RealData;
-impl NumberSpace for RealData {
-    fn is_complex(&self) -> bool {
-        false
-    }
-    fn to_complex(&mut self) {}
-    fn to_real(&mut self) {}
-}
-impl RealNumberSpace for RealData {}
-
-/// Marker for types containing complex data.
-#[derive(Debug, Clone, PartialEq)]
-pub struct ComplexData;
-impl NumberSpace for ComplexData {
-    fn is_complex(&self) -> bool {
-        true
-    }
-    fn to_complex(&mut self) {}
-    fn to_real(&mut self) {}
-}
-impl ComplexNumberSpace for ComplexData {}
-
-/// Marker for types containing real or complex data.
-#[derive(Debug, Clone, PartialEq)]
-pub struct RealOrComplexData {
-    pub is_complex_current: bool,
-}
-impl NumberSpace for RealOrComplexData {
-    fn is_complex(&self) -> bool {
-        self.is_complex_current
-    }
-
-    fn to_complex(&mut self) {
-        self.is_complex_current = true;
-    }
-
-    fn to_real(&mut self) {
-        self.is_complex_current = false;
-    }
-}
-impl RealNumberSpace for RealOrComplexData {}
-impl ComplexNumberSpace for RealOrComplexData {}
-
-/// Marker for types containing time data.
-#[derive(Debug, Clone, PartialEq)]
-pub struct TimeData;
-impl Domain for TimeData {
-    fn domain(&self) -> DataDomain {
-        DataDomain::Time
-    }
-    fn to_freq(&mut self) {}
-    fn to_time(&mut self) {}
-}
-impl TimeDomain for TimeData {}
-
-/// Marker for types containing frequency data.
-#[derive(Debug, Clone, PartialEq)]
-pub struct FrequencyData;
-impl Domain for FrequencyData {
-    fn domain(&self) -> DataDomain {
-        DataDomain::Frequency
-    }
-    fn to_time(&mut self) {}
-    fn to_freq(&mut self) {}
-}
-impl FrequencyDomain for FrequencyData {}
-
-/// Marker for types containing time or frequency data.
-#[derive(Debug, Clone, PartialEq)]
-pub struct TimeOrFrequencyData {
-    pub domain_current: DataDomain,
-}
-impl Domain for TimeOrFrequencyData {
-    fn domain(&self) -> DataDomain {
-        self.domain_current
-    }
-
-    fn to_freq(&mut self) {
-        self.domain_current = DataDomain::Frequency;
-    }
-
-    fn to_time(&mut self) {
-        self.domain_current = DataDomain::Time;
-    }
-}
-
-impl TimeDomain for TimeOrFrequencyData {}
-impl FrequencyDomain for TimeOrFrequencyData {}
-
 /// Expresses at compile time that two classes could potentially represent the same number space or domain.
 pub trait PosEq<O> {}
-impl PosEq<RealData> for RealData {}
-impl PosEq<RealOrComplexData> for RealData {}
-impl PosEq<RealData> for RealOrComplexData {}
-impl PosEq<ComplexData> for RealOrComplexData {}
-impl PosEq<RealOrComplexData> for RealOrComplexData {}
-impl PosEq<ComplexData> for ComplexData {}
-impl PosEq<RealOrComplexData> for ComplexData {}
-
-impl PosEq<TimeData> for TimeData {}
-impl PosEq<TimeOrFrequencyData> for TimeData {}
-impl PosEq<TimeData> for TimeOrFrequencyData {}
-impl PosEq<FrequencyData> for TimeOrFrequencyData {}
-impl PosEq<TimeOrFrequencyData> for TimeOrFrequencyData {}
-impl PosEq<FrequencyData> for FrequencyData {}
-impl PosEq<TimeOrFrequencyData> for FrequencyData {}
 
 /// A 1xN (one times N elements) or Nx1 data vector as used for most digital signal processing
 /// (DSP) operations.
@@ -270,17 +164,6 @@ where
     }
 }
 
-/// Rounds a length so that it always divides by the length of a SIMD
-/// register. This function assumes that `Reg32::LEN > Reg64::LEN`.
-fn round_len(len: usize) -> usize {
-    fn get_reg_len<T: RealNumber, Reg: SimdGeneric<T>>(_: RegType<Reg>) -> usize {
-        Reg::LEN
-    }
-
-    let reg_len = sel_reg!(get_reg_len::<f32>());
-    ((len + reg_len - 1) / reg_len) * reg_len
-}
-
 /// Swaps the halves of two arrays. The `forward` paremeter
 /// specifies what should happen then the `data.len()` is odd.
 /// The function should always produce the same results as `fft_shift`
@@ -301,7 +184,7 @@ where
         let mut pos = step;
         for _ in 0..len {
             let pos_new = (pos + step) % len;
-            unsafe { mem::swap(&mut temp, data.get_unchecked_mut(pos)) };
+            mem::swap(&mut temp, &mut data[pos]);
             pos = pos_new;
         }
     }
@@ -323,6 +206,15 @@ where
     super::transmute_slice_mut(complex)
 }
 
+impl<S: ToSliceMut<T>, T: RealNumber> DspVec<S, T, meta::RealOrComplex, meta::TimeOrFreq> {
+    /// Indicates whether or not the operations on this vector have been successful.
+    /// Consider using the statically typed vector versions so that this check doesn't need to be
+    /// performed.
+    pub fn is_erroneous(&self) -> bool {
+        self.valid_len == 0 && self.delta.is_nan()
+    }
+}
+
 impl<S, T, N, D> DspVec<S, T, N, D>
 where
     S: ToSliceMut<T>,
@@ -330,6 +222,12 @@ where
     N: NumberSpace,
     D: Domain,
 {
+    /// Marks the data of the vector as erroneous
+    fn mark_vector_as_invalid(&mut self) {
+        self.valid_len = 0;
+        self.delta = T::nan();
+    }
+
     /// Executes a real function.
     #[inline]
     fn pure_real_operation<A, F>(&mut self, op: F, argument: A, complexity: Complexity)
@@ -342,7 +240,7 @@ where
             let array = self.data.to_slice_mut();
             Chunk::execute_partial(
                 complexity,
-                self.multicore_settings,
+                &self.multicore_settings,
                 &mut array[0..len],
                 1,
                 argument,
@@ -367,7 +265,7 @@ where
             let array = self.data.to_slice_mut();
             Chunk::execute_partial(
                 complexity,
-                self.multicore_settings,
+                &self.multicore_settings,
                 &mut array[0..len],
                 2,
                 argument,
@@ -413,28 +311,22 @@ where
         {
             let data_length = self.valid_len;
             let array = self.data.to_slice_mut();
-            let (scalar_left, scalar_right, vectorization_length) =
-                Reg::calc_data_alignment_reqs(&array[0..data_length]);
-            if vectorization_length.is_some() {
-                let vectorization_length = vectorization_length.unwrap();
-                Chunk::execute_partial(
-                    complexity,
-                    self.multicore_settings,
-                    &mut array[scalar_left..vectorization_length],
-                    Reg::LEN,
-                    argument,
-                    move |array, argument| {
-                        let array = Reg::array_to_regs_mut(array);
-                        for reg in array {
-                            *reg = simd_op(*reg, argument);
-                        }
-                    },
-                );
-            }
-            for num in &mut array[0..scalar_left] {
-                *num = scalar_op(*num, argument);
-            }
-            for num in &mut array[scalar_right..data_length] {
+            let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
+            Chunk::execute_partial(
+                complexity,
+                &self.multicore_settings,
+                partition.center_mut(array),
+                Reg::LEN,
+                argument,
+                move |array, argument| {
+                    let array = Reg::array_to_regs_mut(array);
+                    for reg in array {
+                        *reg = simd_op(*reg, argument);
+                    }
+                },
+            );
+
+            for num in partition.edge_iter_mut(&mut array[0..data_length]) {
                 *num = scalar_op(*num, argument);
             }
         }
@@ -478,35 +370,23 @@ where
         {
             let data_length = self.valid_len;
             let array = self.data.to_slice_mut();
-            let (scalar_left, scalar_right, vectorization_length) =
-                Reg::calc_data_alignment_reqs(&array[0..data_length]);
-            if vectorization_length.is_some() {
-                let vectorization_length = vectorization_length.unwrap();
-                Chunk::execute_partial(
-                    complexity,
-                    self.multicore_settings,
-                    &mut array[scalar_left..vectorization_length],
-                    Reg::LEN,
-                    argument,
-                    move |array, argument| {
-                        let array = Reg::array_to_regs_mut(array);
-                        for reg in array {
-                            *reg = simd_op(*reg, argument);
-                        }
-                    },
-                );
-            }
-            {
-                let array = array_to_complex_mut(&mut array[0..scalar_left]);
-                for num in array {
-                    *num = scalar_op(*num, argument);
-                }
-            }
-            {
-                let array = array_to_complex_mut(&mut array[scalar_right..data_length]);
-                for num in array {
-                    *num = scalar_op(*num, argument);
-                }
+            let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
+            Chunk::execute_partial(
+                complexity,
+                &self.multicore_settings,
+                partition.center_mut(array),
+                Reg::LEN,
+                argument,
+                move |array, argument| {
+                    let array = Reg::array_to_regs_mut(array);
+                    for reg in array {
+                        *reg = simd_op(*reg, argument);
+                    }
+                },
+            );
+
+            for num in partition.cedge_iter_mut(array_to_complex_mut(&mut array[0..data_length])) {
+                *num = scalar_op(*num, argument);
             }
         }
     }
@@ -532,7 +412,7 @@ where
                 let temp = result.to_slice_mut();
                 Chunk::from_src_to_dest(
                     complexity,
-                    self.multicore_settings,
+                    &self.multicore_settings,
                     &array[0..data_length],
                     2,
                     &mut temp[0..data_length / 2],
@@ -592,45 +472,32 @@ where
         {
             let array = self.data.to_slice_mut();
             let temp = result.to_slice_mut();
-            let (scalar_left, scalar_right, vectorization_length) =
-                Reg::calc_data_alignment_reqs(&array[0..data_length]);
-            if vectorization_length.is_some() {
-                let vectorization_length = vectorization_length.unwrap();
-                Chunk::from_src_to_dest(
-                    complexity,
-                    self.multicore_settings,
-                    &array[scalar_left..vectorization_length],
-                    Reg::LEN,
-                    &mut temp[scalar_left / 2..vectorization_length / 2],
-                    Reg::LEN / 2,
-                    argument,
-                    move |array, range, target, argument| {
-                        let array = Reg::array_to_regs(&array[range.start..range.end]);
-                        let mut j = 0;
-                        for reg in array {
-                            let result = simd_op(*reg, argument);
-                            result.store_half_unchecked(target, j);
-                            j += Reg::LEN / 2;
-                        }
-                    },
-                );
-            }
+            let partition = Reg::calc_data_alignment_reqs(&array[0..data_length]);
+            Chunk::from_src_to_dest(
+                complexity,
+                &self.multicore_settings,
+                partition.center(&array),
+                Reg::LEN,
+                partition.rcenter_mut(temp),
+                Reg::LEN / 2,
+                argument,
+                move |array, range, target, argument| {
+                    let array = Reg::array_to_regs(&array[range.start..range.end]);
+                    let mut j = 0;
+                    for reg in array {
+                        let result = simd_op(*reg, argument);
+                        result.store_half(target, j);
+                        j += Reg::LEN / 2;
+                    }
+                },
+            );
+
+            let array = array_to_complex(&array[0..data_length]);
+            for (src, dest) in partition
+                .cedge_iter(array)
+                .zip(partition.redge_iter_mut(temp))
             {
-                let array = array_to_complex(&array[0..scalar_left]);
-                for pair in array.iter().zip(&mut temp[0..scalar_left / 2]) {
-                    let (src, dest) = pair;
-                    *dest = scalar_op(*src, argument);
-                }
-            }
-            {
-                let array = array_to_complex(&array[scalar_right..data_length]);
-                for pair in array
-                    .iter()
-                    .zip(&mut temp[scalar_right / 2..data_length / 2])
-                {
-                    let (src, dest) = pair;
-                    *dest = scalar_op(*src, argument);
-                }
+                *dest = scalar_op(*src, argument);
             }
 
             self.valid_len /= 2;
@@ -678,7 +545,7 @@ where
                 let converted = convert_mut(&mut data[0..len]);
                 Chunk::execute_with_range(
                     Complexity::Medium,
-                    self.multicore_settings,
+                    &self.multicore_settings,
                     converted,
                     1,
                     function_arg,
@@ -699,7 +566,7 @@ where
                 let converted = convert_mut(&mut data[0..len]);
                 Chunk::execute_sym_pairs_with_range(
                     Complexity::Medium,
-                    self.multicore_settings,
+                    &self.multicore_settings,
                     converted,
                     1,
                     function_arg,
