@@ -15,9 +15,6 @@ use std::ops::{Add, Mul};
 use {array_to_complex, array_to_complex_mut, memcpy, Zero};
 
 /// Provides interpolation operations for real and complex data vectors.
-/// # Unstable
-/// This functionality has been recently added in order to find out if the definitions
-/// are consistent. However the actual implementation is lacking tests.
 pub trait InterpolationOps<S, T>
 where
     S: ToSliceMut<T>,
@@ -48,7 +45,7 @@ where
     ) where
         B: for<'a> Buffer<'a, S, T>;
 
-    /// Interpolates `self` with the convolution function `function` by the interger value
+    /// Interpolates `self` with the convolution function `function` by the integer value
     /// `interpolation_factor`. InterpolationOps is done in in frequency domain.
     ///
     /// See the description of `interpolatef` for some basic performance considerations.
@@ -106,7 +103,7 @@ fn interpolate_priv_scalar<T, TT>(
 {
     Chunk::execute_with_range(
         Complexity::Large,
-        multicore_settings,
+        &multicore_settings,
         temp,
         1,
         data,
@@ -237,77 +234,55 @@ where
             let dest = convert_mut(&mut temp[0..new_len]);
             let len = dest.len();
             let scalar_len = vectors[0].points() * interpolation_factor;
-            let mut i = 0;
-            {
-                let data = convert(&data[0..data_len]);
-                for num in &mut dest[0..scalar_len] {
-                    (*num) = Self::interpolate_priv_simd_step(
-                        i,
-                        interpolation_factor,
-                        conv_len,
-                        data,
-                        &vectors[..],
-                    );
-                    i += 1;
-                }
-            }
 
-            let (scalar_left, _, vectorization_length) =
-                Reg::calc_data_alignment_reqs(&data[0..data_len]);
-            let scalar_left_points = scalar_left / step;
-            if vectorization_length.is_some() {
-                let vectorization_length = vectorization_length.unwrap();
-                let simd = Reg::array_to_regs(&data[scalar_left..vectorization_length]);
-                // Length of a SIMD reg relative to the length of type T
-                // which is 1 for real numbers or 2 for complex numbers
-                let simd_len_in_t = Reg::LEN / step;
-                Chunk::execute_with_range(
-                    Complexity::Large,
-                    self.multicore_settings,
-                    &mut dest[scalar_len..len - scalar_len],
-                    1,
-                    simd,
-                    move |dest_range, range, simd| {
-                        let mut i = range.start + scalar_len;
-                        for num in dest_range {
-                            let rounded = (i + interpolation_factor - 1) / interpolation_factor;
-                            let end = rounded + conv_len - scalar_left_points;
-                            let simd_end = (end + simd_len_in_t - 1) / simd_len_in_t;
-                            let simd_shift = end % simd_len_in_t;
-                            let factor_shift = i % interpolation_factor;
-                            // The reasoning for the next match is analog to the explanation in the
-                            // create_shifted_copies function.
-                            // We need the inverse of the mod unless we start with zero
-                            let factor_shift =
-                                (interpolation_factor - factor_shift) % interpolation_factor;
-                            let selection = factor_shift * simd_len_in_t + simd_shift;
-                            let shifted = &shifts[selection];
-                            let mut sum = Reg::splat(T::zero());
-                            let simd_iter = simd[simd_end - shifted.len()..simd_end].iter();
-                            let iteration = simd_iter.zip(shifted.iter());
-                            for (this, other) in iteration {
-                                sum = sum + simd_mul(*this, *other);
-                            }
-                            (*num) = simd_sum(sum);
-                            i += 1;
+            let partition = Reg::calc_data_alignment_reqs(&data[0..data_len]);
+            let left_points = partition.left / step;
+            let simd = Reg::array_to_regs(partition.center(data));
+            // Length of a SIMD reg relative to the length of type T
+            // which is 1 for real numbers or 2 for complex numbers
+            let simd_len_in_t = Reg::LEN / step;
+            Chunk::execute_with_range(
+                Complexity::Large,
+                &self.multicore_settings,
+                &mut dest[scalar_len..len - scalar_len],
+                1,
+                simd,
+                move |dest_range, range, simd| {
+                    let mut i = range.start + scalar_len;
+                    for num in dest_range {
+                        let rounded = (i + interpolation_factor - 1) / interpolation_factor;
+                        let end = rounded + conv_len - left_points;
+                        let simd_end = (end + simd_len_in_t - 1) / simd_len_in_t;
+                        let simd_shift = end % simd_len_in_t;
+                        let factor_shift = i % interpolation_factor;
+                        // The reasoning for the next match is analog to the explanation in the
+                        // create_shifted_copies function.
+                        // We need the inverse of the mod unless we start with zero
+                        let factor_shift =
+                            (interpolation_factor - factor_shift) % interpolation_factor;
+                        let selection = factor_shift * simd_len_in_t + simd_shift;
+                        let shifted = &shifts[selection];
+                        let mut sum = Reg::splat(T::zero());
+                        let simd_iter = simd[simd_end - shifted.len()..simd_end].iter();
+                        let iteration = simd_iter.zip(shifted.iter());
+                        for (this, other) in iteration {
+                            sum = sum + simd_mul(*this, *other);
                         }
-                    },
-                );
-            }
+                        (*num) = simd_sum(sum);
+                        i += 1;
+                    }
+                },
+            );
 
-            i = len - scalar_len;
-            {
-                let data = convert(&data[0..data_len]);
-                for num in &mut dest[len - scalar_len..len] {
-                    (*num) = Self::interpolate_priv_simd_step(
-                        i,
-                        interpolation_factor,
-                        conv_len,
-                        data,
-                        &vectors[..],
-                    );
-                    i += 1;
-                }
+            let data = convert(&data[0..data_len]);
+            for (i, num) in IndexedEdgeIteratorMut::new(dest, scalar_len, scalar_len) {
+                (*num) = Self::interpolate_priv_simd_step(
+                    i as usize,
+                    interpolation_factor,
+                    conv_len,
+                    data,
+                    &vectors[..],
+                );
             }
         }
         self.valid_len = new_len;
