@@ -121,39 +121,39 @@ where
                 current[k] = T::zero();
                 k += 1;
                 if k >= current.len() {
-                    copy.push(Reg::load_unchecked(&current[..], 0));
+                    copy.push(Reg::load(&current[..], 0));
                     k = 0;
                 }
                 if step > 1 {
                     current[k] = T::zero();
                     k += 1;
                     if k >= current.len() {
-                        copy.push(Reg::load_unchecked(&current[..], 0));
+                        copy.push(Reg::load(&current[..], 0));
                         k = 0;
                     }
                 }
             } else if step > 1 {
-                // Push complex number onto vector
+                // Push complex number into vector
                 let im = *data.next().unwrap();
                 let re = *data.next().unwrap();
                 current[k] = re;
                 k += 1;
                 if k >= current.len() {
-                    copy.push(Reg::load_unchecked(&current[..], 0));
+                    copy.push(Reg::load(&current[..], 0));
                     k = 0;
                 }
                 current[k] = im;
                 k += 1;
                 if k >= current.len() {
-                    copy.push(Reg::load_unchecked(&current[..], 0));
+                    copy.push(Reg::load(&current[..], 0));
                     k = 0;
                 }
             } else {
-                // Push real number onto vector
+                // Push real number into vector
                 current[k] = *data.next().unwrap();
                 k += 1;
                 if k >= current.len() {
-                    copy.push(Reg::load_unchecked(&current[..], 0));
+                    copy.push(Reg::load(&current[..], 0));
                     k = 0;
                 }
             }
@@ -310,7 +310,7 @@ where
                 let conv_len = conv_len as isize;
                 Chunk::execute_with_range(
                     Complexity::Large,
-                    self.multicore_settings,
+                    &self.multicore_settings,
                     dest,
                     1,
                     (complex, other_iter),
@@ -342,7 +342,7 @@ where
                 let conv_len = conv_len as isize;
                 Chunk::execute_with_range(
                     Complexity::Large,
-                    self.multicore_settings,
+                    &self.multicore_settings,
                     dest,
                     1,
                     (data, other_iter),
@@ -528,6 +528,9 @@ where
         }
     }
 
+    /// SIMD optimizatino to convolve two vectors.
+    /// Most of the parameters (convert, convert_mut, simd_mul, simd_sum) are used to have
+    /// a single function for both real valued and complex valued vectors.
     fn convolve_signal_simd_impl<Reg, B, TT, O, C, CMut, RMul, RSum>(
         &mut self,
         buffer: &mut B,
@@ -549,12 +552,7 @@ where
         let points = self.points();
         let other_points = vector.points();
         assert!(other_points < points);
-        let (other_start, other_end, full_conv_len, conv_len) = (
-            0,
-            other_points,
-            other_points,
-            other_points - other_points / 2,
-        );
+        let (full_conv_len, conv_len) = (other_points, other_points - other_points / 2);
         let len = self.len();
         let mut temp = buffer.borrow(len);
         {
@@ -565,14 +563,43 @@ where
             let other = convert(&other[0..vector.len()]);
             let complex = convert(&data[0..len]);
             let dest = convert_mut(&mut temp[0..len]);
-            let other_iter = &other[other_start..other_end];
+            let other_iter = &other[0..other_points];
 
             let shifts = create_shifted_copies(vector);
 
             // The next lines uses + $reg::LEN due to rounding of odd numbers
             let scalar_len = conv_len + Reg::LEN;
             let conv_len = conv_len as isize;
-            for (i, num) in dest[0..scalar_len].iter_mut().enumerate() {
+
+            let partition = Reg::calc_data_alignment_reqs(&data[0..len]);
+            let step = if self.is_complex() { 2 } else { 1 };
+            let scalar_left_points = partition.left / step;
+            let simd = Reg::array_to_regs(partition.center(data));
+            Chunk::execute_with_range(
+                Complexity::Large,
+                &self.multicore_settings,
+                &mut dest[scalar_len..points - scalar_len],
+                1,
+                simd,
+                move |dest_range, range, simd| {
+                    let mut i = (scalar_len + range.start) as isize;
+                    for num in dest_range {
+                        let end = (i + conv_len) as usize - scalar_left_points;
+                        let shift = end % shifts.len();
+                        let end = (end + shifts.len() - 1) / shifts.len();
+                        let mut sum = Reg::splat(T::zero());
+                        let shifted = &shifts[shift];
+                        let simd_iter = simd[end - shifted.len()..end].iter();
+                        let iteration = simd_iter.zip(shifted.iter());
+                        for (this, other) in iteration {
+                            sum = sum + simd_mul(*this, *other);
+                        }
+                        (*num) = simd_sum(sum);
+                        i += 1;
+                    }
+                },
+            );
+            for (i, num) in IndexedEdgeIteratorMut::new(dest, scalar_len, scalar_len) {
                 *num = Self::convolve_iteration(
                     complex,
                     other_iter,
@@ -581,45 +608,6 @@ where
                     full_conv_len,
                 );
             }
-
-            let (scalar_left, _, vectorization_length) =
-                Reg::calc_data_alignment_reqs(&data[0..len]);
-            let step = if self.is_complex() { 2 } else { 1 };
-            let scalar_left_points = scalar_left / step;
-            if vectorization_length.is_some() {
-                let vectorization_length = vectorization_length.unwrap();
-                let simd = Reg::array_to_regs(&data[scalar_left..vectorization_length]);
-                Chunk::execute_with_range(
-                    Complexity::Large,
-                    self.multicore_settings,
-                    &mut dest[scalar_len..points - scalar_len],
-                    1,
-                    simd,
-                    move |dest_range, range, simd| {
-                        let mut i = (scalar_len + range.start) as isize;
-                        for num in dest_range {
-                            let end = (i + conv_len) as usize - scalar_left_points;
-                            let shift = end % shifts.len();
-                            let end = (end + shifts.len() - 1) / shifts.len();
-                            let mut sum = Reg::splat(T::zero());
-                            let shifted = &shifts[shift];
-                            let simd_iter = simd[end - shifted.len()..end].iter();
-                            let iteration = simd_iter.zip(shifted.iter());
-                            for (this, other) in iteration {
-                                sum = sum + simd_mul(*this, *other);
-                            }
-                            (*num) = simd_sum(sum);
-                            i += 1;
-                        }
-                    },
-                );
-            }
-
-            let mut i = (points - scalar_len) as isize;
-            for num in &mut dest[points - scalar_len..points] {
-                *num = Self::convolve_iteration(complex, other_iter, i, conv_len, full_conv_len);
-                i += 1;
-            }
         }
         temp.trade(&mut self.data);
     }
@@ -646,7 +634,7 @@ where
             let converted = convert_mut(&mut data[0..len]);
             Chunk::execute_with_range(
                 Complexity::Medium,
-                self.multicore_settings,
+                &self.multicore_settings,
                 converted,
                 1,
                 (ratio, function_arg),
@@ -670,7 +658,7 @@ where
             let points = converted.len();
             Chunk::execute_sym_pairs_with_range(
                 Complexity::Medium,
-                self.multicore_settings,
+                &self.multicore_settings,
                 converted,
                 1,
                 (ratio, function_arg),
@@ -681,7 +669,7 @@ where
                     let scale = TT::from(ratio);
                     let len1 = array1.len();
                     let len2 = array2.len();
-                    let offset = if points % 2 != 0 { 1 } else { 0 };
+                    let offset = points % 2;
                     let max = T::from(points - offset).unwrap() / two;
                     let center = T::from(points - offset).unwrap() / two;
                     let mut j1 = -center + T::from(range1.start).unwrap();
@@ -721,7 +709,7 @@ where
                     // in the previous loop.
                     let pos1 = len1 - i1;
                     let pos2 = len2 - i2;
-                    let common_length = if pos1 < pos2 { pos1 } else { pos2 };
+                    let common_length = pos1.min(pos2);
                     for num in &mut array1[i1 + common_length..len1] {
                         (*num) =
                             (*num) * scale * fun(arg, fft_swap_x(is_fft_shifted, j1, max) * ratio);
