@@ -41,8 +41,6 @@ where
 
     /// Performs a Fast Fourier Transformation transforming a time domain vector
     /// into a frequency domain vector.
-    /// # Unstable
-    /// FFTs of real vectors are unstable.
     /// # Example
     ///
     /// ```
@@ -89,9 +87,6 @@ where
     ///
     /// 1. `VectorMustBeReal`: if `self` is in complex number space.
     /// 2. `VectorMustBeInTimeDomain`: if `self` is in frequency domain.
-    ///
-    /// # Unstable
-    /// Symmetric IFFTs are unstable and may only work under certain conditions.
     fn plain_sfft<B>(self, buffer: &mut B) -> TransRes<Self::FreqResult>
     where
         B: for<'a> Buffer<'a, S, T>;
@@ -104,9 +99,6 @@ where
     ///
     /// 1. `VectorMustBeReal`: if `self` is in complex number space.
     /// 2. `VectorMustBeInTimeDomain`: if `self` is in frequency domain.
-    ///
-    /// # Unstable
-    /// Symmetric IFFTs are unstable and may only work under certain conditions.
     fn sfft<B>(self, buffer: &mut B) -> TransRes<Self::FreqResult>
     where
         B: for<'a> Buffer<'a, S, T>;
@@ -119,9 +111,6 @@ where
     ///
     /// 1. `VectorMustBeReal`: if `self` is in complex number space.
     /// 2. `VectorMustBeInTimeDomain`: if `self` is in frequency domain.
-    ///
-    /// # Unstable
-    /// Symmetric IFFTs are unstable and may only work under certain conditions.
     fn windowed_sfft<B>(
         self,
         buffer: &mut B,
@@ -146,7 +135,7 @@ where
         B: for<'a> Buffer<'a, S, T>,
     {
         if self.domain() != DataDomain::Time {
-            self.valid_len = 0;
+            self.mark_vector_as_invalid();
             self.number_space.to_complex();
             self.domain.to_freq();
             return Self::FreqResult::rededicate_from_force(self);
@@ -208,7 +197,7 @@ where
         B: for<'a> Buffer<'a, S, T>,
     {
         if self.domain() != DataDomain::Time || self.is_complex() {
-            self.valid_len = 0;
+            self.mark_vector_as_invalid();
             self.number_space.to_complex();
             self.domain.to_freq();
             return Err((
@@ -218,7 +207,7 @@ where
         }
 
         if self.points() % 2 == 0 {
-            self.valid_len = 0;
+            self.mark_vector_as_invalid();
             self.number_space.to_complex();
             self.domain.to_freq();
             return Err((
@@ -240,7 +229,7 @@ where
         B: for<'a> Buffer<'a, S, T>,
     {
         if self.domain() != DataDomain::Time || self.is_complex() {
-            self.valid_len = 0;
+            self.mark_vector_as_invalid();
             self.number_space.to_complex();
             self.domain.to_freq();
             return Err((
@@ -250,7 +239,7 @@ where
         }
 
         if self.points() % 2 == 0 {
-            self.valid_len = 0;
+            self.mark_vector_as_invalid();
             self.number_space.to_complex();
             self.domain.to_freq();
             return Err((
@@ -276,7 +265,7 @@ where
         B: for<'a> Buffer<'a, S, T>,
     {
         if self.domain() != DataDomain::Time || self.is_complex() {
-            self.valid_len = 0;
+            self.mark_vector_as_invalid();
             self.number_space.to_complex();
             self.domain.to_freq();
             return Err((
@@ -286,7 +275,7 @@ where
         }
 
         if self.points() % 2 == 0 {
-            self.valid_len = 0;
+            self.mark_vector_as_invalid();
             self.number_space.to_complex();
             self.domain.to_freq();
             return Err((
